#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = { "../../../font/Vera.ttf,size=18" };
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_box_t *root, *row;
	mbtk_label_t *lab;
	mbtk_button_t *but1, *but2;

	example_init_sdl2(&argc, &argv);

	/* create a window */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk boxing example", -1, -1, 300, 200);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root box as a vbox */
	root = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &root->w);

	/* first row: label, button, button */
	row = mbtk_hbox_new(NULL);
	lab = mbtk_label_new(NULL, "Normal:");
	but1 = mbtk_button_new_with_label("hello");
	but2 = mbtk_button_new_with_label("world!");

	mbtk_box_add_widget(root, &row->w, 0);
	mbtk_box_add_widget(row, &lab->w, 0);
	mbtk_box_add_widget(row, &but1->w, 0);
	mbtk_box_add_widget(row, &but2->w, 0);

	/* next row: same, but right align */
	row = mbtk_hbox_new(NULL);
	lab = mbtk_label_new(NULL, "Right:");
	but1 = mbtk_button_new_with_label("hello");
	but2 = mbtk_button_new_with_label("world!");

	mbtk_box_add_widget(root, &row->w, 0);
	mbtk_box_add_widget(row, &lab->w, 0);
	mbtk_box_add_widget(row, &but1->w, 0);
	mbtk_box_add_widget(row, &but2->w, 0);

	mbtk_box_set_align(row, HVBOX_RIGHT_BOTTOM);


	/* next row: left align, with more spacing between the widgets */
	row = mbtk_hbox_new(NULL);
	lab = mbtk_label_new(NULL, "Left+spacing:");
	but1 = mbtk_button_new_with_label("hello");
	but2 = mbtk_button_new_with_label("world!");

	mbtk_box_add_widget(root, &row->w, 0);
	mbtk_box_add_widget(row, &lab->w, 0);
	mbtk_box_add_widget(row, &but1->w, 0);
	mbtk_box_add_widget(row, &but2->w, 0);

	mbtk_box_set_align(row, HVBOX_LEFT_TOP);
	mbtk_box_set_spacing(row, 14);


	for(;;) {
		mbtk_event_t ev;
		mbtk_iterate(&disp, &ev, 1);
	}

	/* Uninit everyhting at once */
	mbtk_uninit();
}
