#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = { "../../../font/Vera.ttf,size=18" };
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

static int button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	/* user_data is what mbtk_button_callback_click() had as 3rd parameter */
	fprintf(stderr, "Button clicked\n");
	return 0;
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_box_t *root;
	mbtk_label_t *lab;
	mbtk_button_t *btn;

	example_init_sdl2(&argc, &argv);

	/* create a small window */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk hello world", -1, -1, 150, 50);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root widget as a vbox so child widgets are packed vertically;
	   default alignment is "justify", which means excess space are put between
	   the widgets. */
	root = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &root->w);

	/* Create and add a label */
	lab = mbtk_label_new(NULL, "Hello world!");
	mbtk_box_add_widget(root, &lab->w, 0);

	/* Create an add a button */
	btn = mbtk_button_new_with_label("Click me!");
	mbtk_button_callback_click(btn, button_clk, NULL);
	mbtk_box_add_widget(root, &btn->w, 0);

	/* Run the main loop forever */
	for(;;) {
		mbtk_event_t ev;
		mbtk_iterate(&disp, &ev, 1);
	}

	/* Uninit everyhting at once */
	mbtk_uninit();
}
