#define INCLUDE_SELECT
#include "../../config.h"
#include "fdwatch.h"

void mbtk_watch_fd_set(mbtk_watch_fd_t *ctx, int fd, int for_read, int for_write)
{
	long n, rdi = -1, wri = -1;
	int need_bump = 0, need_recalc = 0;

	for(n = 0; n < ctx->rd.used; n++)
		if (ctx->rd.array[n] == fd)
			rdi = n;

	for(n = 0; n < ctx->wr.used; n++)
		if (ctx->wr.array[n] == fd)
			wri = n;


	if (for_read && (rdi < 0)) {
		vtl0_append(&ctx->rd, fd);
		need_bump = 1;
	}
	else if (!for_read && (rdi >= 0)) {
		vtl0_remove(&ctx->rd, rdi, 1);
		need_recalc = 1;
	}

	if (for_write && (wri < 0)) {
		vtl0_append(&ctx->wr, fd);
		need_bump = 1;
	}
	else if (!for_write && (wri >= 0)) {
		vtl0_remove(&ctx->wr, wri, 1);
		need_recalc = 1;
	}

	if (need_recalc) { /* after any removal a full recalculation is necessary */
		ctx->last = -1;
		for(n = 0; n < ctx->rd.used; n++)
			if (ctx->rd.array[n] > ctx->last)
				ctx->last = ctx->rd.array[n];
		for(n = 0; n < ctx->wr.used; n++)
			if (ctx->wr.array[n] > ctx->last)
				ctx->last = ctx->wr.array[n];
	}
	else if (need_bump) { /* alternatively, after adding a new, just bump */
		if (fd > ctx->last)
			ctx->last = fd;
	}

}

int mbtk_watch_fd_select(mbtk_watch_fd_t *ctx, mbtk_event_t *dst, long *usec, mbtk_window_t *win)
{
	fd_set rd, wr;
	struct timeval tv;
	int sr;
	long n;

	FD_ZERO(&rd);
	FD_ZERO(&wr);

	for(n = 0; n < ctx->rd.used; n++)
		FD_SET(ctx->rd.array[n], &rd);

	for(n = 0; n < ctx->wr.used; n++)
		FD_SET(ctx->wr.array[n], &wr);

	tv.tv_sec = 0;
	tv.tv_usec = *usec;
	sr = select(ctx->last+1, &rd, &wr, NULL, &tv);

	*usec = tv.tv_usec;

	if (sr <= 0)
		return sr;

	for(n = 0; n < ctx->rd.used; n++) {
		if (FD_ISSET(ctx->rd.array[n], &rd)) {
			dst->type = MBTK_EV_FDWATCH;
			dst->win = win;
			dst->data.fdwatch.fd = ctx->rd.array[n];
			dst->data.fdwatch.readable = 1;
			dst->data.fdwatch.writable = 0;
			dst->data.fdwatch.closed = 0;
			return 1;
		}
	}

	for(n = 0; n < ctx->wr.used; n++) {
		if (FD_ISSET(ctx->wr.array[n], &rd)) {
			dst->type = MBTK_EV_FDWATCH;
			dst->win = win;
			dst->data.fdwatch.fd = ctx->wr.array[n];
			dst->data.fdwatch.readable = 0;
			dst->data.fdwatch.writable = 1;
			dst->data.fdwatch.closed = 0;
			return 1;
		}
	}

	return 0;
}

void mbtk_watch_fd_uninit(mbtk_watch_fd_t *ctx)
{
	vtl0_uninit(&ctx->rd);
	vtl0_uninit(&ctx->wr);
	ctx->last = -1;
}
