#include <stdlib.h>

#define INCLUDE_TIME_SUBSEC
#include "../../config.h"

#include "monotime.h"

#ifdef MBTK_HAVE_CLOCK_MONOTONIC_RAW
#	define MBTK_MONOTONIC_CLOCK CLOCK_MONOTONIC_RAW
#else
#	define MBTK_MONOTONIC_CLOCK CLOCK_MONOTONIC
#endif

double mbtk_be_get_monotime(void)
{
#if defined(MBTK_HAVE_CLOCK_GETTIME)
	struct timespec now;
	clock_gettime(MBTK_MONOTONIC_CLOCK, &now);
	return (double)now.tv_sec + (double)(now.tv_nsec) / 1000000000.0;
#elif defined(MBTK_HAVE_MACH_ABSOLUTE_TIME)
	uint64_t now = mach_absolute_time();
	return ((double)((now * mach_base_info.numer) / mach_base_info.denom) / 1000000000.0);
#elif defined(MBTK_HAVE_TIMEGETTIME)
	return (doubel)timeGetTime() / 1000.0;
#elif defined(MBTK_HAVE_QUERYPERFORMANCECOUNTER)
	static LARGE_INTEGER freq = 0;
	LARGE_INTEGER now;
	if (freq == 0)
		QueryPerformanceFrequency(&freq);
	return (double)now.QuadPart / (double)hires_ticks_per_second.QuadPart;
#elif defined(MBTK_HAVE_GETTIMEOFDAY)
	struct timeval now;
	gettimeofday(&now, NULL);
	return (double)(now.tv_sec) + (double)(now.tv_usec) / 1000000.0;
#elif defined(MBTK_HAVE_FTIME)
	struct timeb tb;
	ftime(&tb);
	return (double)tb.time + (double)(tb.millitm) / 1000.0;
#else
	mbtk_error("mbtk_be_get_monotime(): No time source available on this system.");
	abort();
#endif
	return -1;
}
