static char mbtk_xlib_mouse_button_xlate(int button)
{
	switch(button) {
		case 1: return MBTK_MB_LEFT;
		case 2: return MBTK_MB_MIDDLE;
		case 3: return MBTK_MB_RIGHT;
	}
	return MBTK_MB_UNKNOWN;
}

#define SCROLL(dx, dy) \
{ \
	if (event.type != ButtonRelease) break; \
	dst->type = MBTK_EV_MOUSE_SCROLL; \
	dst->data.mouse.x = dx; \
	dst->data.mouse.y = dy; \
	return 1; \
}

static mbtk_keymod_t mbtk_xlib_key_mods(unsigned int state)
{
	mbtk_keymod_t res = 0;
	if (state & ShiftMask)   res |= MBTK_KM_SHIFT;
	if (state & Mod1Mask)    res |= MBTK_KM_ALT;
	if (state & ControlMask) res |= MBTK_KM_CTRL;
	return res;
}

#ifndef XK_MISCELLANY
#define XK_MISCELLANY
#endif

#include <X11/keysymdef.h>
#include <X11/Xutil.h>

static mbtk_keysym_t mbtk_xlib_key_sym(XEvent event, mbtk_keymod_t *mod)
{
	KeySym ks = XLookupKeysym(&event.xkey, 0);

	if ((ks >= 32) && (ks <= 127))
		return ks;

	switch(ks) {
		case XK_BackSpace:        return MBTK_KS_BACKSPACE;
		case XK_Tab:              return MBTK_KS_TAB;
		case XK_KP_Enter:         (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_ENTER;
		case XK_Return:           return MBTK_KS_ENTER;
		case XK_Escape:           return MBTK_KS_ESC;

		case XK_F1:               return MBTK_KS_FN+1;
		case XK_F2:               return MBTK_KS_FN+2;
		case XK_F3:               return MBTK_KS_FN+3;
		case XK_F4:               return MBTK_KS_FN+4;
		case XK_F5:               return MBTK_KS_FN+5;
		case XK_F6:               return MBTK_KS_FN+6;
		case XK_F7:               return MBTK_KS_FN+7;
		case XK_F8:               return MBTK_KS_FN+8;
		case XK_F9:               return MBTK_KS_FN+9;
		case XK_F10:              return MBTK_KS_FN+10;
		case XK_F11:              return MBTK_KS_FN+11;
		case XK_F12:              return MBTK_KS_FN+12;
		case XK_F13:              return MBTK_KS_FN+13;
		case XK_F14:              return MBTK_KS_FN+14;
		case XK_F15:              return MBTK_KS_FN+15;
		case XK_F16:              return MBTK_KS_FN+16;
		case XK_F17:              return MBTK_KS_FN+17;
		case XK_F18:              return MBTK_KS_FN+18;
		case XK_F19:              return MBTK_KS_FN+19;
		case XK_F20:              return MBTK_KS_FN+20;
		case XK_F21:              return MBTK_KS_FN+21;
		case XK_F22:              return MBTK_KS_FN+22;
		case XK_F23:              return MBTK_KS_FN+23;
		case XK_F24:              return MBTK_KS_FN+24;
		case XK_F25:              return MBTK_KS_FN+25;
		case XK_F26:              return MBTK_KS_FN+26;
		case XK_F27:              return MBTK_KS_FN+27;
		case XK_F28:              return MBTK_KS_FN+28;
		case XK_F29:              return MBTK_KS_FN+29;
		case XK_F30:              return MBTK_KS_FN+30;
		case XK_F31:              return MBTK_KS_FN+31;
		case XK_F32:              return MBTK_KS_FN+32;
		case XK_F33:              return MBTK_KS_FN+33;
		case XK_F34:              return MBTK_KS_FN+34;
		case XK_F35:              return MBTK_KS_FN+35;

		case XK_Home:             return MBTK_KS_HOME;
		case XK_KP_Home:          (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_HOME;
		case XK_End:              return MBTK_KS_END;
		case XK_KP_End:           (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_END;
		case XK_Left:             return MBTK_KS_LEFT;
		case XK_KP_Left:          (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_LEFT;
		case XK_Right:            return MBTK_KS_RIGHT;
		case XK_KP_Right:         (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_RIGHT;
		case XK_Up:               return MBTK_KS_UP;
		case XK_KP_Up:            (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_UP;
		case XK_Down:             return MBTK_KS_DOWN;
		case XK_KP_Down:          (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_DOWN;
		case XK_Page_Up:          return MBTK_KS_PGUP;
		case XK_KP_Page_Up:       (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_PGUP;
		case XK_Page_Down:        return MBTK_KS_PGDN;
		case XK_KP_Page_Down:     (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_PGDN;

		case XK_Shift_L:          return MBTK_KS_SHIFT_L;
		case XK_Shift_R:          return MBTK_KS_SHIFT_R;
		case XK_Alt_L:            return MBTK_KS_ALT_L;
		case XK_Alt_R:            return MBTK_KS_ALT_R;
		case XK_Control_L:        return MBTK_KS_CTRL_L;
		case XK_Control_R:        return MBTK_KS_CTRL_R;

		case XK_Delete:           return MBTK_KS_DEL;
		case XK_KP_Delete:        (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_DEL;
		case XK_Insert:           return MBTK_KS_INS;
		case XK_KP_Insert:        (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_INS;
	}

	return MBTK_KS_INVALID;
}

#include "../../config.h"

#ifdef MBTK_HAVE_KEYSYMTOUCS4
extern unsigned int KeySymToUcs4(KeySym keysym);
#endif

static void mbtk_xlib_key_xlate(mbtk_event_t *dst, XEvent event)
{
	dst->data.key.scancode = event.xkey.keycode;
	dst->data.key.mods = mbtk_xlib_key_mods(event.xkey.state);
	dst->data.key.sym = mbtk_xlib_key_sym(event, &dst->data.key.mods);

	if (dst->type == MBTK_EV_KEY_RELEASE) {
		KeySym keysym;
		XLookupString(&event.xkey, NULL, 0, &keysym, NULL);
#ifdef MBTK_HAVE_KEYSYMTOUCS4
		dst->data.key.edit = KeySymToUcs4(keysym);
#else
		dst->data.key.edit = keysym;
#endif
	}
	else
		dst->data.key.edit = 0;
}
