#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "hvbox.h"

static void dump_span(FILE *f, hvbox_span_t span)
{
	if (span == HVBOX_MINIMAL)
		fprintf(f, "min");
	else if (span == HVBOX_FILL)
		fprintf(f, "fill");
	else
		fprintf(f, "%d%%", span);
}

static void dump_align(FILE *f, hvbox_align_t a, char *c)
{
	if ((a >= 0) && (a <= HVBOX_RIGHT_BOTTOM))
		fprintf(f, "%c", c[a]);
	else
		fprintf(f, "?");
}

void hvbox_dump_node(FILE *f, int level, hvbox_t *node, int widget_is_str)
{
	char ind[128];
	char *stype;

	assert(level < sizeof(ind)-1);
	if (level > 0)
		memset(ind, ' ', level);
	ind[level] = '\0';

	switch(node->type) {
		case HVBOX_ROOT: stype = "root"; break;
		case HVBOX_HBOX: stype = "hbox"; break;
		case HVBOX_VBOX: stype = "vbox"; break;
		case HVBOX_WIDGET: stype = "widget"; break;
		default: stype = "???";
	}

	fprintf(f, "%s%s min=%dx%d (%dx%d) %s=%dx%d at %d;%d", ind, stype, node->w_min, node->h_min, node->w_min_calc, node->h_min_calc, (node->dirty ? "CALC" : "calc"), node->w, node->h, node->x, node->y);

	fprintf(f, " span=");
	dump_span(f, node->span);

	fprintf(f, " align=");
	if (node->type == HVBOX_HBOX)
		dump_align(f, node->align, "#.<>");
	else if (node->type == HVBOX_VBOX)
		dump_align(f, node->align, "#.^v");
	else
		fprintf(f, "n/a");

	if (node->hidden)
		fprintf(f, "H");
	else if (node->hidden_calc)
		fprintf(f, "h");
	else
		fprintf(f, " ");

	if (widget_is_str)
		fprintf(f, " w='%s'", (char *)node->widget);
	else
		fprintf(f, " w=%p", node->widget);

	fprintf(f, "\n");


}

void hvbox_dump_nodes(FILE *f, int level, hvbox_t *node, int widget_is_str)
{
	hvbox_t *n;

	hvbox_dump_node(f, level, node, widget_is_str);
	for(n = node->ch_first; n != NULL; n = n->next)
		hvbox_dump_nodes(f, level+1, n, widget_is_str);
}

#if 0
/* calculated */
	int w_avail, h_avail;      /* available space left after packing up all children */
#endif
