#include <libmbtk/config.h>
#include <stdlib.h>
#include <string.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/draw.h>

/*** For implementing widgets **/

/* Create a new widget w from an user allocation or by caling calloc() */
#define mbtk_create_wt(widget, type_kw, imps, caller_alloced_) \
	do { \
		if (caller_alloced != NULL) { \
			widget = caller_alloced_; \
			memset(widget, 0, sizeof(*widget)); \
			widget->w.flags.caller_alloced = 1; \
		} \
		else \
			widget = calloc(sizeof(*widget), 1); \
		widget->w.struct_size = sizeof(*widget); \
		widget->w.type = type_kw; \
		widget->w.hvbox.type = HVBOX_WIDGET; \
		widget->w.imp = imps; \
		widget->w.flags.dirty = 1; \
		widget->w.flags.created = 1; \
	} while(0)

mbtk_widget_t *mbtk_widget_pre_insert(mbtk_widget_t *w, mbtk_widget_t *parent);
void mbtk_widget_post_insert(mbtk_widget_t *w);
void mbtk_widget_minsize(mbtk_widget_t *widget, long *mx, long *my);

/* Call this if previously floating widget got embedded under a parent */
MBTK_INLINE void mbtk_widget_embedded(mbtk_widget_t *parent, mbtk_widget_t *w)
{
	long w_min, h_min;

	if (w->win == parent->win)
		return;
	
	w->win = parent->win;
	if (w->win != NULL)
		mbtk_win_dirty_layout(w->win);

	w = mbtk_widget_pre_insert(w, parent);
	mbtk_widget_minsize(w, &w_min, &h_min);
	w->hvbox.w_min = w_min;
	w->hvbox.h_min = h_min;

}

/* Look up color by kw_clr in w's style and set drawing color for w->win if
   found. Returns 0 on success, -1 if not found */
MBTK_INLINE int mbtk_widget_draw_set_style_color(mbtk_widget_t *w, mbtk_kw_t kw_clr)
{
	mbtk_arg_t hica;

	if ((mbtk_widget_style_get_prop(w, kw_clr, &hica) == 0) && (hica.type == MBTK_ULONG)) {
		int r, g, b, a;
		r = (hica.v.ul & 0xff0000000ul) >> 24;
		g = (hica.v.ul & 0x000ff0000ul) >> 16;
		b = (hica.v.ul & 0x00000ff00ul) >> 8;
		a = (hica.v.ul & 0x0000000fful);
		mbtk_draw_color(w->win, r, g, b, a);
		return 0;
	}
	return -1;
}
