#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include "../be_init.h"

static char **font;
static int stay;

static int button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Button click: '%s'\n", (const char *)user_data);
	return 0;
}

static int button_toggle(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Toggle: '%s' %d\n", (const char *)user_data, w->flags.activated);
	return 0;
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *vbox, *w1, *w2, *hbox1;
	mbtk_label_t *wl, *bl;
	mbtk_button_t *wb;
	mbtk_tbutton_t *wtb;
	mbtk_checkbox_t *wch;
	mbtk_radiobutton_t *wrb;
	mbtk_handler_radio_t *hrr;
	mbtk_progbar_t *wprb;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);


	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

/*	mbtk_style_set_prop(disp.style, disp.styleprop, mbtk_kw("frame.width"), mbtk_arg_long(2));
	mbtk_style_set_prop(disp.style, disp.styleprop, mbtk_kw("frame.width"), mbtk_arg_long_mult(3));*/

/*	mbtk_display_style_set_prop(&disp, mbtk_kw("frame.width"), mbtk_arg_long(2));
	mbtk_display_style_set_prop(&disp, mbtk_kw("frame.width"), mbtk_arg_long_mult(3));*/

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk simple widgets", -1, -1, 300, 300);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	vbox = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &vbox->w);

	w1 = mbtk_vbox_new(NULL);
	w2 = mbtk_vbox_new(NULL);
	wl = mbtk_label_new(NULL, "hello\nworld!");
	bl = mbtk_label_new(NULL, "click me");
	wb = mbtk_button_new(NULL);
	mbtk_button_set_child(wb, &bl->w);
	mbtk_button_callback_click(wb, button_clk, "blobb");

	wtb = mbtk_tbutton_new_with_label("toggle me");
	mbtk_tbutton_callback_toggle(wtb, button_toggle, "buttonbarf");

	mbtk_box_add_widget(vbox, &w1->w, 0);
	mbtk_box_add_widget(vbox, &wl->w, 0);
	mbtk_box_add_widget(vbox, &wb->w, 0);
	mbtk_box_add_widget(vbox, &wtb->w, 0);

	/*** Checkbox button ***/

	/* create and append a hbox; this will host our example radio buttons */
	hbox1 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	/* create a checkbox; preserve default handler (which is "toggle") */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf1");
	mbtk_box_add_widget(hbox1, &wch->w, 0);

	/* create another checkbox */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf2");
	mbtk_box_add_widget(hbox1, &wch->w, 0);


	/* create another checkbox */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf3");
	mbtk_box_add_widget(hbox1, &wch->w, 0);
	mbtk_checkbox_style_set_prop(wch, mbtk_kw("fg.color"), mbtk_arg_str("#ff0000")); /* change the foreground color, which is the color of the mark */


	/*** Radio button ***/

	/* create and append a hbox; this will host our example radio buttons */
	hbox1 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	/* create a radio handler within the current widow */
	hrr = mbtk_handler_radio_alloc(win);

	/* create a radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 1");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_radio_add_widget(hrr, &wrb->w); /* add to the radio handler; this groups our radio buttons */

	/* create a second radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 2");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_radio_add_widget(hrr, &wrb->w);

	/* create a third button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 3");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_radiobutton_style_set_prop(wrb, mbtk_kw("fg.color"), mbtk_arg_str("#ff0000")); /* change the foreground color, which is the color of the mark */
	mbtk_handler_radio_add_widget(hrr, &wrb->w);

	/*** progress bar ***/
	wprb = mbtk_progbar_new(NULL);
	mbtk_box_add_widget(vbox, &wprb->w, 0);
	wprb->curr = 0.33;

	mbtk_box_add_widget(vbox, &w2->w, 0);

	mbtk_box_set_align(vbox, HVBOX_JUSTIFY);
	mbtk_box_set_spacing(vbox, 4);
	mbtk_box_set_span(w1, 20);
	mbtk_box_set_span(w2, HVBOX_FILL);

	mbtk_box_set_padding(vbox, 14);
	mbtk_box_set_padding(w1, 14);
	mbtk_box_set_padding(w2, 14);


	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_iterate(&disp, &ev, 1)) {
/*			printf("ev %d\n", ev.type);*/
			if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;
			}
		}
	}

	mbtk_display_uninit(&be, &disp);
	mbtk_be_uninit(&be);
}
